#pragma once

extern MIDIFILE	g_mf;

class Event {
public:
	virtual void Save(FILE *fp)
	{
		SaveTCT(fp);
	};
	void SaveTCT(FILE *fp) { // Type Channel Time, 16 bit
		u8 tc = (type&15) << 4 | channel&15;
		fwrite(&time, 1, 1, fp);
		fwrite(&tc, 1, 1, fp);
	}

	u32		type;
	u32		time; // absolute MIDI time or relative SAD time
	u8		channel;
	
};

struct EventComp {
	bool operator()(Event* l, Event* r) {
		return (l->time < r->time) || (l->time == r->time) && (l->type != E_NOTE_ON) && (r->type == E_NOTE_ON);
	}
};

class EvEnd : public Event {
public:
	EvEnd() {
		type = E_END;
		time = 0;
		channel = 0;
	}
};


class EvContinue : public Event {
public:
	EvContinue() {
		type = E_CONTINUE;
		time = 255;
		channel = 0;
	}

};

class EvTempo : public Event {
public:
	EvTempo() {
		channel = g_mf.Status & 0x0F;
		time = g_mf.Time;
		type = E_TEMPO;
	}
	void Set(u32 data) {
		mpqn = data;
	}
	void Save(FILE *fp) // Tempo is implicit
	{
	};

	u32	mpqn;
};

class EvControl : public Event {
public:
	u8 ctrl;
};

class EvProgram : public Event {
public:
	EvProgram() {
		type = E_PROGRAM;
		time = g_mf.Time;
		channel = g_mf.Status & 0x0F;
		program = g_mf.Data[0];
	}
	void Save(FILE *fp) {
		SaveTCT(fp);
		fwrite(&program, 1, 1, fp);
	}
	u8 program;
};

class EvPitch : public Event {
public:
	EvPitch() {
		type = C_PITCH;
		time = g_mf.Time;
		channel = g_mf.Status & 0x0F;
		pitch = g_mf.Data[1];
	}
	void Save(FILE *fp) {
		SaveTCT(fp);
		fwrite(&pitch, 2, 1, fp);
	}
	u16 pitch;
};

u16 semitone2offset(float semitone);
u16 semitoneDif2offset(float s1, float s2);

class EvNoteOn : public Event {
public:
	EvNoteOn() {
		type = E_NOTE_ON;
		time = g_mf.Time;
		channel = g_mf.Status & 0x0F;
		pitch = g_mf.Data[0];
		vol = g_mf.Data[1];
	}
	void Save(FILE *fp) {
		if(type == E_NOTE_ON_PORTAM)
			type = C_PITCH;
		SaveTCT(fp);
		if(type == E_NOTE_ON) {
			fwrite(&vol, 1, 1, fp);
		}
		fwrite(&pitch, 2, 1, fp);
	}
	u16 pitch;
	u8 vol;
};

class EvNoteOff : public Event {
public:
	EvNoteOff() {
		type = E_NOTE_OFF;
		time = g_mf.Time;
		channel = g_mf.Status & 0x0F;
		pitch = g_mf.Data[0];
	}
	void Save(FILE *fp) {
		SaveTCT(fp);
		//fwrite(&pitch, 1, 1, fp);
	}
	u16 pitch;
};

class EvMidiControl : public EvControl {
public:
	EvMidiControl() {
		type = E_CONTROL;
		channel = g_mf.Status & 0x0F;
		time = g_mf.Time;
		ctype = g_mf.Data[0];
		ctrl = 2*g_mf.Data[1];
	}
	void Save(FILE *fp) {
		type = ctype;
		SaveTCT(fp);
		fwrite(&ctrl, 1, 1, fp);
	}
	u8 ctype;
};

class EvRepeat : public Event {
public:
	EvRepeat() {
		type = C_REPEAT;
		time = g_mf.Time;
		channel = g_mf.Status & 0x0F;
		times = g_mf.Data[1];
	}
	void Save(FILE *fp) {
		SaveTCT(fp);
		fwrite(&times, 1, 1, fp);
	}
	u8 times;
};

class EvEnvelope : public Event {
public:
	void Set(u8 t, u8 i, u8 a, u8 d, u8 h) {
		type = t;
		initial = i;
		attack = a;
		decay = d;
		hold = h;
	}

	void Save(FILE *fp) {
		SaveTCT(fp);
		fwrite(&init_state, 1, 1, fp);
		switch(init_state) {
			case ENV_ATTACK:
				fwrite(&initial, 1, 1, fp);
				fwrite(&attack, 1, 1, fp);
			case ENV_DECAY:
				fwrite(&decay, 1, 1, fp);
			case ENV_HOLD:
				fwrite(&hold, 1, 1, fp);
		}
	}
	u8	init_state;
	u8	initial;
	u8	attack;
	u8	decay;
	u8	hold;
};

typedef std::vector<Event*> EvVec;

struct Channel {
	size_t	NumEvents() {
		return events.size();
	}
	EvVec	events;
	u8		volume;
};

extern Channel g_channels[NUM_CHANNELS];