// Rouslan Dimitrov 2011. This is free software - use it to learn or modify it to suit your needs.
#define _CRT_SECURE_NO_DEPRECATE
#define WIN32_LEAN_AND_MEAN
#pragma warning (disable:4996)
#pragma warning (disable:4244)
#pragma comment(lib, "midifile.lib")

#include <windows.h>
#include <iostream>
#include <vector>
#include <algorithm>
#include <math.h>
#include "midifile.h"
#include "../synth/declarations.h"

using namespace std;

MIDICALL	g_cb;
MIDIFILE	g_mf;

#include "events.h" // reduce clutter
#include "filters.h"


Channel g_channels[NUM_CHANNELS];

const float	DT = 1.0f / SAMPLE_RATE;

long APIENTRY readEvent(MIDIFILE * mf)
{
	u8 ichan = g_mf.Status & 0xF;
	if(ichan > NUM_CHANNELS - 1)
		return 0;

	Channel *chan = &g_channels[ ichan ];

	switch ( (mf->Status >> 4) & 0xF ) {
		case E_NOTE_ON:
			if(mf->Data[1]) // vol != 0
				chan->events.push_back( new EvNoteOn() );
			else
				chan->events.push_back( new EvNoteOff() );
			break;
		case E_NOTE_OFF:
			chan->events.push_back( new EvNoteOff() );
			break;
		case E_CONTROL:
			switch(g_mf.Data[0]) {
				case C_PITCH: chan->events.push_back( new EvPitch() ); break;
				case C_REPEAT: chan->events.push_back( new EvRepeat() ); break;
				case C_MOD: // Only channel 0 supports modulation
					if((g_mf.Status & 0xF) != 0)
						break;
				//case C_FM_STRENGTH:
				case C_PAN:
				case C_BLEND: chan->events.push_back( new EvMidiControl() ); break;
			}
			break;
		//case E_PROGRAM:
		//	if(FM_SYNTH || g_mf.Data[0] != 1)
		//		chan->events.push_back( new EvProgram() );
		//	break;
	}
	return 0;
}

long APIENTRY readTempo(METATEMPO * mf)
{
	EvTempo *e = new EvTempo;
	e->Set(mf->Tempo);
	g_channels[0].events.push_back(e);

	return 0;
}

void AddEnvelopes(EvVec& events, char *song) {
	char* fExt = song+strlen(song);
	strcpy(fExt, ".env");
	FILE *fp = fopen(song, "rt");
	*fExt = 0;

	if(!fp) {
		cout << "Can't open " << song << endl;
		return;
	}
	char buffer[256];
	EvEnvelope *env = new EvEnvelope;
	while(fgets(buffer, 255, fp)) {
		char type[8];
		if(buffer[0] == '#' || buffer[0] == '\n')
			continue;
		size_t n = sscanf(buffer, "%d %d %8s %1s %d %d %d %d\n", &env->time, &env->channel, type, &env->init_state, &env->initial, &env->attack, &env->decay, &env->hold);
		if(n == 8) {
			env->type = (type[0] == 'V' || type[0] == 'v') ? E_VOL_ENV_SET : E_BL_ENV_SET;
			switch(env->init_state) {
				case 'A': env->init_state = ENV_ATTACK; break;
				case 'D': env->init_state = ENV_DECAY; break;
				case 'H': env->init_state = ENV_HOLD; break;
			}
			events.push_back(env);
			env = new EvEnvelope;
		}
	}
	delete env;
	fclose(fp);
}

bool loadWave(char *filename, s8* wave)
{
	s16 wave16[WAVET_SIZE];
	FILE *fp = fopen(filename, "rb");

	if(!fp) {
		cout << "Can't open " << filename << endl;
		return 0;
	}

	fread(wave16, 2, WAVET_SIZE, fp);

	for(int i=0; i<WAVET_SIZE; i++) {
		s32 t = wave16[i];					// cast to s32
		wave[i] = (u8)(t >> 8);
	}

	return !fclose(fp);
}

bool writeSadFile(char *song, EvVec &allEvents)
{
	s8 wave[2*WAVET_SIZE];
	char fileName[64];

	strcpy(fileName, song);
	char* fExt = fileName+strlen(fileName);

	for(u32 i=0; i<NUM_WAVES; i++) {

		fExt[0] = '-';
		fExt[1] = '0' + i;
		strcpy(fExt+2, ".raw");

		if(!loadWave(fileName, wave + i*WAVET_SIZE))
			return 0;
	}

	strcpy(fExt, ".sad");
	FILE *fp = fopen(fileName, "wb");

	if(!fp)
		return 0;

	fwrite("sad1", 4, 1, fp); // header

	for(u32 i=0; i<NUM_WAVES; i++) {
		if(i) for(u32 j=0; j<WAVET_SIZE; j++) {
			wave[i*WAVET_SIZE + j] -= wave[j];
		}

		fwrite(wave + i*WAVET_SIZE, 1, WAVET_SIZE, fp); // wave table
	}
	
	for(size_t i=0; i<allEvents.size(); i++)
		allEvents[i]->Save(fp);

	cout << fileName << " written.\n";

	return !fclose(fp);
}

u16 semitone2offset(float semitone)
{
    float freq = 440.0f * pow(2.0f, (semitone - 69.0f)/12.0f);
    return (u16)(WAVET_SIZE*freq*DT*512 + 0.5f);			// scale by 512 to increase resolution while still fitting in u16
}

u16 semitoneDif2offset(float s1, float s2)
{
    float freq = 440.0f * (pow(2.0f, (s1 - 69.0f)/12.0f) - pow(2.0f, (s2 - 69.0f)/12.0f));
	float r = freq*DT*256.0*16.0*WAVET_SIZE;
    return r+0.5f;			// scale by 256*16 to increase resolution while still fitting in u16
}

int __cdecl main(int argc, char *argv[])
{
	if ( argc < 2 ) {
		cout << "Usage:\n sconv midi_file\n\nThis converter understands only notes, tempo and pitchbend.";
		return E_USAGE;
	}

	size_t filename_len = strlen(argv[1]);
	if(filename_len < 4) {
		cout << "Expecting a .mid file\n";
		return E_USAGE;
	}

	memset(&g_cb, 0, sizeof(g_cb));

	typedef long (APIENTRY *StandardEvt)();
	typedef long (APIENTRY *MetaTempo)();

	*reinterpret_cast<void**>(&g_cb.StandardEvt) = &readEvent;
	*reinterpret_cast<void**>(&g_cb.MetaTempo) = &readTempo;

	g_mf.Handle = (HANDLE)argv[1];
	g_mf.MidiCalls = &g_cb;

	int midiBad = MidiReadFile(&g_mf);
	if(midiBad) {
		u8 buf[80];
		MidiGetErr(&g_mf, buf, 80, midiBad);
		cout << buf << endl;
		return E_MIDI;
	}

	// Remove .mid extension
	*(argv[1]+filename_len-4) = 0;

	EvVec allEvents;
	allEvents.reserve(8192);

	for(int i=0; i<NUM_CHANNELS; i++) {
		Channel *chan = &g_channels[i];
		//sort(chan->events.begin(), chan->events.end(), EventComp()); // seem to be in order from the midi file

		RemoveRedundantOffEvents(chan->events);
		ComputeSampleOffsets(chan->events);
		CompressNoteVolumeAndPan(chan->events);

		allEvents.insert(allEvents.end(), chan->events.begin(), chan->events.end());
	}

	AddEnvelopes(allEvents, argv[1]);

	sort(allEvents.begin(), allEvents.end(), EventComp());

	ConvertTimeToSampleDeltas(allEvents);

	// Mark the end of song
	allEvents.push_back(new EvEnd);
	allEvents.back()->time = 0;
	if(!writeSadFile(argv[1], allEvents))
		return E_WRITE;

	return E_OK;
}
