#pragma once

#ifdef _MSC_VER
#define inline
#include <stdio.h>
#else
#include <avr/io.h>
#endif

// Synth Declarations

#define	WAVET_SIZE		64
#define SAMPLE_RATE		32000
#define	CONTROL_RATE	100
#define NUM_CHANNELS	4
#define NUM_WAVES		2
#define CPU_SPEED		20000000
#define F_CPU			CPU_SPEED

enum event_types {
	E_CONTINUE	= 0x0,
	E_NOTE_OFF	= 0x8,
	E_NOTE_ON	= 0x9,
	E_NOTE_ON_LAST_VOL	= 0xA,
	E_NOTE_ON_PORTAM	= 0x1A,
	E_CONTROL	= 0xB,
	E_PROGRAM	= 0xC,
	E_END,
	E_VOL_ENV_SET,
	E_BL_ENV_SET,
	E_TEMPO		= 0x51,
};

enum control_types {	// .sad control types; must not overlap with the above
	C_MOD		= 0x1,
	C_PITCH		= 0x2,
	C_REPEAT,
	C_BLEND,
	C_PAN		= 0xA
};

enum envelop_state {
	ENV_ATTACK,
	ENV_DECAY,
	ENV_HOLD,
	ENV_FORCED = ENV_HOLD
};

enum ret_vals {
	E_OK,
	E_USAGE,
	E_MIDI,
	E_WRITE,
	E_AUDIO
};

#ifdef _M_IX86
#define SIMULATOR 1
#else
#define SIMULATOR 0
#endif

#if SIMULATOR
typedef unsigned int	u32;
typedef unsigned short	u16;
typedef unsigned char	u8;
typedef int				s32;
typedef short			s16;
typedef char			s8;

#else
#include <stdint.h>
typedef uint32_t	u32;
typedef uint16_t	u16;
typedef uint8_t		u8;
typedef int32_t		s32;
typedef int16_t		s16;
typedef int8_t		s8;
#endif

// LCD Declarations

#if SIMULATOR
static char LCD_DATAPORT;
static char LCD_CMDPORT;
static char UI_BUTTONPORT;
static char PORTC;
static char ADCH;
static char ADCSRA;
#else
#define LCD_DATAPORT	PORTD
#define LCD_CMDPORT		PORTC
#define UI_BUTTONPORT	PINB
#endif
// MSVSC doesn't like the 0b prefix
#define LCD_COMMAND		0	//0b0		// PORTC
#define LCD_CHAR		1	//0b1
#define LCD_ENABLE		2	//0b10
#define LCD_DISABLE		0	//0b00
#define LCD_LOCSEL		16	//0b10000
#define LCD_ON			24	//0b00011000 // PORTD
#define LCD_2LINE		120	//0b01111000
#define LCD_CLEAR		2	//0b00000010
#define LCD_SCROLL_L	48	//0b00110000
#define LCD_SCROLL_R	56	//0b00111000
#define LCD_SEL_LINE2	64	//0x40
#define LCD_LAST_CHAR	87	//0x57
#define LCD_INCREMENT	12	//0b1100
#define LCD_DECREMENT	8	//0b1000 // Prints in reverse


// One of the next 3 is called, followed by lcd_update per slow clock
// Printing state can be changed only if the lcd is not busy
void lcd_print_value(u8 val);
void lcd_print_menu(char *str, u8 val);
void lcd_init(char *str, u8 val);

void lcd_update();
u8 lcd_busy();

// UI Declarations

#define BUTTON_LEFT		8	//0b1000
#define BUTTON_RIGHT	1	//0b0001

#define LED_RED			4	//0b0100	// PORTC
#define LED_GREEN		8	//0b1000

void menu_update();

