// Rouslan Dimitrov 2011. This is free software - use it to learn or modify it to suit your needs.
#include "..\synth\declarations.h"

inline void send_char1(char c) {
#if SIMULATOR
	putchar(c);
#endif
	LCD_CMDPORT  = LCD_DISABLE | LCD_CHAR;
	LCD_DATAPORT = c << 1;
}

inline void send_char2() {
	LCD_CMDPORT  = LCD_ENABLE | LCD_CHAR;
}

inline void send_cmd1(u8 c) {
	LCD_CMDPORT  = LCD_DISABLE | LCD_COMMAND;
	LCD_DATAPORT = c;
}

inline void send_cmd2() {
	LCD_CMDPORT = LCD_ENABLE | LCD_COMMAND;
}

inline void send_cmd_cursor_pos(u8 loc) {
	LCD_CMDPORT  = LCD_DISABLE | LCD_COMMAND | LCD_LOCSEL;
	LCD_DATAPORT = loc << 1;
}

inline void send_cmd_cursor_pos2() {
	LCD_CMDPORT  = LCD_ENABLE | LCD_COMMAND | LCD_LOCSEL;
}

char *string;
u8 value;
u8 cycle;
enum {
	LCD_IDLE,
	LCD_STRING,
	LCD_VALUE,
	LCD_INIT,
};
u8 mode;

u8 lcd_busy()
{
	return mode;
}

// number is printed in reverse
void lcd_print_value(u8 val)
{
	cycle = 0;
	value = val;
	mode = LCD_VALUE;
}

inline void atoi()
{
	if(value > 0) {
		send_char1(value % 10 + '0');
		value /= 10;
	} else {
		send_char1(' ');
	}
}

inline void print_value() {
	if(cycle > 8) {
		mode = LCD_IDLE;
		return;
	}
	switch(cycle) {
		case 0:	send_cmd_cursor_pos(LCD_LAST_CHAR); break;
		case 1:	send_cmd_cursor_pos2(); break;
		case 2:	send_cmd1(LCD_DECREMENT); break;
		case 3:	send_cmd2(); break;
		case 4:
			atoi();
			break;
		case 5:
		case 6:
			send_char2();
			atoi();
			break;
		case 7:
			send_char2();
			send_char1(' ');
			break;
		case 8:
			send_char2();
			break;
	}
	cycle++;
}


void lcd_print_menu(char *str, u8 val)
{
	cycle = 0;
	string = str;
	value = val;
	mode = LCD_STRING;
}

inline void print_string() {
	if(cycle > 47) {
		mode = LCD_VALUE;
		cycle = 0;
		return;
	}
	switch(cycle) {
		case 0:	send_cmd1(LCD_CLEAR);//send_cmd_cursor_pos(0); break;
		case 1:	send_cmd2(); break;
		case 2:	send_cmd1(LCD_INCREMENT); break;
		case 3:	send_cmd2(); break;
		default:
			if(cycle & 1)
				send_char2();
			else {
				u8 c = string[(cycle-4)>>1];
				if(c != 0)
					send_char1(c);
				else
					cycle = 46;
			} break;
	}
	cycle++;
}

void lcd_init(char *str, u8 val)
{
	cycle = 0;
	string = str;
	value = val;
	mode = LCD_INIT;
}

inline void init()
{
	if(cycle > 3) {
		mode = LCD_STRING;
		cycle = 0;
		return;
	}
	switch(cycle) {
		case 0:	send_cmd1(LCD_2LINE); break;
		case 1:	send_cmd2(); break;
		case 2:	send_cmd1(LCD_ON); break;
		case 3:	send_cmd2(); break;
	}
	cycle++;
}

void lcd_update()
{
	switch(mode) {
		case LCD_IDLE:
			return;
		case LCD_VALUE: print_value(); break;
		case LCD_STRING: print_string(); break;	
		case LCD_INIT: init(); break;
	}
}
