// Rouslan Dimitrov 2011. This is free software - use it to learn or modify it to suit your needs.
#define _CRT_SECURE_NO_DEPRECATE
#define WIN32_LEAN_AND_MEAN
#include <windows.h> // for sleep
#include <stdio.h>
#include <SDL/SDL.h>
#include "declarations.h"

#pragma warning (disable:4996)
#pragma comment(lib, "sdl.lib")

u8 *song_data;
s8 wave_table1[WAVET_SIZE];	// soft
s8 wave_table2[WAVET_SIZE];	// harsh
FILE *out_file;

void control();
void generate_sample(u8 *out);
void synth_main();

int load_sad(void *filename)
{
	size_t file_len;
	u32 header;
	FILE *fp = fopen(filename, "rb");

	if(!fp)
		return -1;

	fseek(fp, 0, SEEK_END);
	file_len = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	fread(&header, 4, 1, fp);

	if(header != *(u32*)"sad1")
		return -1;

	file_len -= WAVET_SIZE*NUM_WAVES;

	song_data = malloc(file_len);

	fread(wave_table1, WAVET_SIZE, 1, fp);
	fread(wave_table2, WAVET_SIZE, 1, fp);

	fread(song_data, file_len, 1, fp);

	return fclose(fp);
}

void fill_audio(void *udata, Uint8 *data, int len)
{
	int i;
	for( i=0; i < len; i+=2 ) {
		generate_sample(data+i);
		//printf("%4d %4d\n", data[i], data[i+1]);
	}
	if(out_file) {
		fwrite(data, 1, len, out_file);			
		fflush(out_file);
	}
}

#undef main
int main(int argc, char *argv[])
{
	SDL_AudioSpec wanted;
	
	wanted.freq		= SAMPLE_RATE;
	wanted.format	= AUDIO_U8;
	wanted.channels	= 2;
	wanted.samples	= 4096;
	wanted.callback = fill_audio;
	wanted.userdata = 0;
	
	if ( argc < 2 ) {
		puts("Usage: synth sad_file [out_file]\n");
		return E_USAGE;
	} 

	if (load_sad(argv[1]) < 0) {
		puts("Couldn't load sad file.");
		return E_MIDI;
	}

	if(argc == 3) {
		out_file = fopen(argv[2], "wb");
	}

	SDL_Init(SDL_INIT_AUDIO);

    if ( SDL_OpenAudio(&wanted, NULL) < 0 ) {
        puts(SDL_GetError());
        return E_AUDIO;
    }

	SDL_PauseAudio(0);

	synth_main();
	puts("Playing...\n");

	Sleep(INFINITE);
	
	SDL_CloseAudio();
	free(song_data);

	if(argc == 3)
		fclose(out_file);

    return E_OK;
}