// Rouslan Dimitrov 2011. This is free software - use it to learn or modify it to suit your needs.
#include "declarations.h"

#if SIMULATOR
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif

enum menu_items {
	M_VERSION,
	M_VOLUME,
	M_CONTROL_RATE,
	M_NUMITEMS
};

u8 menu_item;
u8 menu_changed;
u8 left_state;
u8 right_state;
u8 slider_state;
extern u8 volume;
extern u16 trigger_val;
u8 control_rate = CONTROL_RATE;
extern char *version_string;

void menu_update()
{
#if SIMULATOR
	left_state  = left_state  << 1 | (GetKeyState(VK_LEFT) & 0x80 ? 1:0);
	right_state = right_state << 1 | (GetKeyState(VK_RIGHT) & 0x80 ? 1:0);
#else
	left_state  = left_state  << 1 | (UI_BUTTONPORT & BUTTON_LEFT  ? 1:0);
	right_state = right_state << 1 | (UI_BUTTONPORT & BUTTON_RIGHT ? 1:0);
#endif
	if((left_state & 0xF) == 0x8) {
		menu_item--;
		if(menu_item >= M_NUMITEMS)
			menu_item = M_NUMITEMS - 1;
		menu_changed = 1;
	}

	if((right_state & 0xF) == 0x8) {
		menu_item++;
		if(menu_item >= M_NUMITEMS)
			menu_item = 0;
		menu_changed = 1;
	}

	if(!lcd_busy()) {
		u8 slider = ADCH;
		if(slider != slider_state) {
			PORTC |= LED_GREEN;
			slider_state = slider;
			switch(menu_item) {
				case M_VERSION: break;
				case M_VOLUME:
					volume = slider;
					lcd_print_value(volume);
					break;
				case M_CONTROL_RATE:
					control_rate = slider;
					trigger_val = SAMPLE_RATE / control_rate;
					lcd_print_value(control_rate);
					break;
			}
		}
	}

	ADCSRA |= 0x40;//0b01000000;

	if(menu_changed) {
		PORTC |= LED_RED;

		switch(menu_item) {
			case M_VERSION:			lcd_print_menu(version_string, 0); break;
			case M_VOLUME:			lcd_print_menu("Volume", volume); break;
			case M_CONTROL_RATE:	lcd_print_menu("Control Rate", control_rate); break; 
		}

		menu_changed = 0;
	}
}


